@@ -13,6 +13,7 @@ from simplesat.pool import Pool
 from simplesat.repository import Repository
 from simplesat.request import Request
 
+from fusesoc.capi2.coreparser import Core2Parser
 from fusesoc.core import Core
 from fusesoc.librarymanager import LibraryManager
 
@@ -35,11 +36,11 @@ class CoreDB:
 
     # simplesat doesn't allow ':', '-' or leading '_'
     def _package_name(self, vlnv):
-        _name = "{}_{}_{}".format(vlnv.vendor, vlnv.library, vlnv.name).lstrip("_")
+        _name = f"{vlnv.vendor}_{vlnv.library}_{vlnv.name}".lstrip("_")
         return _name.replace("-", "__")
 
     def _package_version(self, vlnv):
-        return "{}-{}".format(vlnv.version, vlnv.revision)
+        return f"{vlnv.version}-{vlnv.revision}"
 
     def _parse_depend(self, depends):
         # FIXME: Handle conflicts
@@ -56,6 +57,13 @@ class CoreDB:
                 )
         return ", ".join(deps)
 
+    def _parse_virtual(self, virtuals):
+        package_names = []
+        for virtual in virtuals:
+            for simple in virtual.simpleVLNVs():
+                package_names.append("{}".format(self._package_name(simple)))
+        return ", ".join(package_names)
+
     def add(self, core, library):
         self._solver_cache_invalidate_all()
 
@@ -63,7 +71,7 @@ class CoreDB:
         logger.debug("Adding core " + name)
         if name in self._cores:
             _s = "Replacing {} in {} with the version found in {}"
-            logger.debug(
+            logger.warning(
                 _s.format(name, self._cores[name]["core"].core_root, core.core_root)
             )
         self._cores[name] = {"core": core, "library": library}
@@ -124,18 +132,37 @@ class CoreDB:
         cores = [x["core"] for x in self._cores.values()]
         for core in cores:
             if only_matching_vlnv:
-                if not eq_vln(core.name, top_core):
+                if not any(
+                    [eq_vln(core.name, top_core)]
+                    + [
+                        eq_vln(virtual_vlnv, top_core)
+                        for virtual_vlnv in core.get_virtuals(_flags)
+                    ]
+                ):
                     continue
 
+            # Build a "pretty" package string in a format expected by
+            # PrettyPackageStringParser()
             package_str = "{} {}-{}".format(
-                self._package_name(core.name), core.name.version, core.name.revision
+                self._package_name(core.name),
+                core.name.version,
+                core.name.revision,
             )
+
+            _virtuals = core.get_virtuals(_flags)
+            if _virtuals:
+                _s = "; provides ( {} )"
+                package_str += _s.format(self._parse_virtual(_virtuals))
+
+            # Add dependencies only if we want to build the whole dependency
+            # tree.
             if not only_matching_vlnv:
                 _flags["is_toplevel"] = core.name == top_core
                 _depends = core.get_depends(_flags)
                 if _depends:
                     _s = "; depends ( {} )"
                     package_str += _s.format(self._parse_depend(_depends))
+
             parser = PrettyPackageStringParser(EnpkgVersion.from_string)
 
             package = parser.parse_to_package(package_str)
@@ -144,14 +171,12 @@ class CoreDB:
             repo.add_package(package)
 
         request = Request()
-        simplevlnvs = top_core.simpleVLNVs()
-        for sv in simplevlnvs:
-            _top_dep = "{} {} {}".format(
-                self._package_name(top_core),
-                top_core.relation,
-                self._package_version(top_core),
-            )
-            request.install(Requirement._from_string(_top_dep))
+        _top_dep = "{} {} {}".format(
+            self._package_name(top_core),
+            top_core.relation,
+            self._package_version(top_core),
+        )
+        request.install(Requirement._from_string(_top_dep))
 
         installed_repository = Repository()
         pool = Pool([repo])
@@ -166,13 +191,10 @@ class CoreDB:
             raise DependencyError(top_core.name)
 
         objdict = {}
-        depdict = {}
         if len(transaction.operations) > 1:
             for op in transaction.operations:
-                objdict[op.package._name] = str(op.package.core.name)
-                depdict[str(op.package.core.name)] = [
-                    objdict[n[0]] for n in op.package.install_requires
-                ]
+                for p in op.package.provides:
+                    objdict[p[0]] = str(op.package.core.name)
                 op.package.core.direct_deps = [
                     objdict[n[0]] for n in op.package.install_requires
                 ]
@@ -185,28 +207,74 @@ class CoreDB:
 
 
 class CoreManager:
-    def __init__(self, config):
+    def __init__(self, config, library_manager=None):
         self.config = config
         self.db = CoreDB()
-        self._lm = LibraryManager(config.library_root)
+        self._lm = (
+            LibraryManager(config.library_root)
+            if library_manager == None
+            else library_manager
+        )
+        self.core2parser = Core2Parser(
+            config.resolve_env_vars_early, config.allow_additional_properties
+        )
 
-    def find_cores(self, library):
+    def find_cores(self, library, ignored_dirs):
         found_cores = []
         path = os.path.expanduser(library.location)
         exclude = {".git"}
         if os.path.isdir(path) == False:
             raise OSError(path + " is not a directory")
         logger.debug("Checking for cores in " + path)
+        visited = set()
         for root, dirs, files in os.walk(path, followlinks=True):
-            if "FUSESOC_IGNORE" in files:
+            ignore_tree = ("FUSESOC_IGNORE" in files) or (
+                os.path.abspath(root) in ignored_dirs
+            )
+            if ignore_tree:
                 del dirs[:]
                 continue
-            dirs[:] = [directory for directory in dirs if directory not in exclude]
+
+            keep_dirs = []
+            for _d in dirs:
+                # Ignore sub dirs in the exclude set
+                if _d in exclude:
+                    continue
+
+                st = os.stat(os.path.join(root, _d))
+                dirkey = st.st_dev, st.st_ino
+                # Ignore dirs we already visited. Protects against endless symlink recursion
+                if dirkey in visited:
+                    continue
+
+                visited.add(dirkey)
+                keep_dirs.append(_d)
+
+            dirs[:] = keep_dirs
+
             for f in files:
                 if f.endswith(".core"):
                     core_file = os.path.join(root, f)
                     try:
-                        core = Core(core_file, self.config.cache_root)
+                        capi_version = self._detect_capi_version(core_file)
+                        if capi_version == 1:
+                            # Skip core files which are not in CAPI2 format.
+                            logger.error(
+                                "Core file {} is in CAPI1 format, which is not supported "
+                                "any more since FuseSoC 2.0. The core file is ignored. "
+                                "Please migrate your cores to the CAPI2 file format, or "
+                                "use FuseSoC 1.x as stop-gap.".format(core_file)
+                            )
+                            continue
+                        elif capi_version == -1:
+                            # Skip core files which are not FuseSoc format at all.
+                            continue
+
+                        core = Core(
+                            parser=self.core2parser,
+                            core_file=core_file,
+                            cache_root=self.config.cache_root,
+                        )
                         found_cores.append(core)
                     except SyntaxError as e:
                         w = "Parse error. Ignoring file " + core_file + ": " + e.msg
@@ -214,15 +282,55 @@ class CoreManager:
                     except ImportError as e:
                         w = 'Failed to register "{}" due to unknown provider: {}'
                         logger.warning(w.format(core_file, str(e)))
+                    except ValueError as e:
+                        logger.warning(e)
         return found_cores
 
-    def _load_cores(self, library):
-        found_cores = self.find_cores(library)
+    def _detect_capi_version(self, core_file) -> int:
+        """Detect the CAPI version in a .core file
+
+        Returns:
+            Version of the core file (1 or 2)
+        """
+        try:
+            with open(core_file) as f:
+                l = f.readline().split()
+                if l:
+                    first_line = l[0]
+                else:
+                    first_line = ""
+                if first_line == "CAPI=1":
+                    return 1
+                elif first_line == "CAPI=2:":
+                    return 2
+                else:
+                    error_msg = (
+                        "The first line of the core file {} must be "
+                        ' "CAPI=1" or "CAPI=2:".'.format(core_file)
+                    )
+                    error_msg += '  The first line of this core file is "{}".'.format(
+                        first_line
+                    )
+                    if first_line == "CAPI=2":
+                        error_msg += "  Just add a colon on the end!"
+                    logger.warning(error_msg)
+                    raise ValueError(
+                        "Unable to determine CAPI version from core file {}.".format(
+                            core_file
+                        )
+                    )
+        except Exception as error:
+            error_msg = f"Unable to determine CAPI version from core file {core_file}"
+            logger.warning(error_msg)
+            return -1
+
+    def _load_cores(self, library, ignored_dirs):
+        found_cores = self.find_cores(library, ignored_dirs)
         for core in found_cores:
             self.db.add(core, library)
 
-    def add_library(self, library):
-        """ Register a library """
+    def add_library(self, library, ignored_dirs):
+        """Register a library"""
         abspath = os.path.abspath(os.path.expanduser(library.location))
         _library = self._lm.get_library(abspath, "location")
         if _library:
@@ -230,11 +338,11 @@ class CoreManager:
             logger.warning(_s.format(library.name, abspath, _library.name))
             return
 
-        self._load_cores(library)
+        self._load_cores(library, ignored_dirs)
         self._lm.add_library(library)
 
     def get_libraries(self):
-        """ Get all registered libraries """
+        """Get all registered libraries"""
         return self._lm.get_libraries()
 
     def get_depends(self, core, flags):
@@ -259,21 +367,21 @@ class CoreManager:
         return deps
 
     def get_cores(self):
-        """ Get a dict with all cores, indexed by the core name """
+        """Get a dict with all cores, indexed by the core name"""
         return {str(x.name): x for x in self.db.find()}
 
     def get_core(self, name):
-        """ Get a core with a given name """
+        """Get a core with a given name"""
         c = self.db.find(name)
         c.name.relation = "=="
         return c
 
     def get_generators(self):
-        """ Get a dict with all registered generators, indexed by name """
+        """Get a dict with all registered generators, indexed by name"""
         generators = {}
         for core in self.db.find():
             if hasattr(core, "get_generators"):
-                _generators = core.get_generators({})
+                _generators = core.get_generators()
                 if _generators:
                     generators[str(core.name)] = _generators
         return generators
